unit TrayIcons;

interface

uses
  Windows, Messages, Classes, Controls, ShellAPI, Graphics;

type
  TMouseEvent = procedure(Sender: TObject; Button: TMouseButton) of object;

  TTrayIcon = class(TComponent)
  private
    FWindowHandle: HWND;
    FIconData: TNotifyIconData;
    FIcon: TIcon;
    FHint: String;
    FOnEvent: array[0..2] of TMouseEvent;
    procedure SetIcon(const Value: TIcon);
    procedure SetHint(const Value: String);
    procedure SetStruct;
    procedure CreateTrayIcon;
    procedure ModifyTrayIcon;
    procedure DeleteTrayIcon;
    procedure WndProc(var Msg: TMessage);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Icon: TIcon read FIcon write SetIcon;
    property Hint: String read FHint write SetHint;
    property OnMouseDown: TMouseEvent read FOnEvent[0] write FOnEvent[0];
    property OnMouseUp: TMouseEvent read FOnEvent[1] write FOnEvent[1];
    property OnDblClick: TMouseEvent read FOnEvent[2] write FOnEvent[2];
  end;

implementation

uses
  SysUtils;

const
  WM_TRAYICON = WM_USER + 2001;
  
constructor TTrayIcon.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FWindowHandle := AllocateHWnd(WndProc);
  FIcon := TIcon.Create;
  CreateTrayIcon;
end;

destructor TTrayIcon.Destroy;
begin
  DeleteTrayIcon;
  FIcon.Free;
  DeallocateHWnd(FWindowHandle);
  inherited Destroy;
end;

procedure TTrayIcon.SetIcon(const Value: TIcon);
begin
  FIcon.Assign( Value );
  ModifyTrayIcon;
end;

procedure TTrayIcon.SetHint(const Value: String);
begin
  FHint := Value;
  ModifyTrayIcon;
end;

procedure TTrayIcon.WndProc(var Msg: TMessage);
var
  Button: TMouseButton;
  EventIndex: Integer;
begin
  with Msg do
   case Msg of
   WM_TRAYICON: begin
                  if ( LParam >= WM_LBUTTONDOWN ) and ( LParam <= WM_MBUTTONDBLCLK ) then
                   begin
                     Button := TMouseButton( ( LParam - WM_LBUTTONDOWN ) div 3 );
                     EventIndex := ( LParam - WM_LBUTTONDOWN ) - ( Ord( Button ) * 3 );

                     SetForegroundWindow( FWindowHandle ); 
                     if Assigned( FOnEvent[ EventIndex ] ) then
                      FOnEvent[ EventIndex ]( Self, Button );
                   end;
                end;
   else Result := DefWindowProc(FWindowHandle, Msg, wParam, lParam);
   end;
end;

procedure TTrayIcon.SetStruct;
begin
  with FIconData do
   begin
     cbSize := SizeOf( TNotifyIconData );
     Wnd := FWindowHandle;
     uID := FWindowHandle;
     hIcon := Icon.Handle;
     StrPCopy( szTip, FHint );
     uCallbackMessage := WM_TRAYICON;
     uFlags := NIF_MESSAGE + NIF_ICON + NIF_TIP;
   end;
end;

procedure TTrayIcon.CreateTrayIcon;
begin
  SetStruct;
  Shell_NotifyIcon( NIM_ADD, @FIconData );
end;

procedure TTrayIcon.ModifyTrayIcon;
begin
  SetStruct;
  Shell_NotifyIcon( NIM_MODIFY, @FIconData );
end;

procedure TTrayIcon.DeleteTrayIcon;
begin
  Shell_NotifyIcon( NIM_DELETE, @FIconData );
end;

end.
