unit uCtxPuppyReader;

interface

uses Classes,ComCtrls,XMLIntf,ActiveX,IdHTTP,XMLDoc,SysUtils,StrUtils,Forms,SyncObjs;

const ALIAS = 'PuppyReader';
const SITES_FILE = 'Sites.xml';
const CACHE_DIR = 'cache';
const ROOT_NODE = 'FeedData';
const ITEM_NODE = 'FeedItem';

const RSS_ROOT = 'rss';
const RSS_TITLE = 'Title';
const RSS_DESC = 'Description';
const RSS_WEB_URL = 'WebPageUrl';
const RSS_IMAGE = 'Image';
const RSS_IMAGE_URL = 'Url';
const RSS_LINK = 'Link';
const RSS_IMAGE_WIDTH = 'Width';
const RSS_IMAGE_HEIGHT = 'Height';
const RSS_ITEM = 'Items';
//PubDate  Ⱦ ѵ ־ date ϴ  Ѵ.
const RSS_PUBDATE = 'date';
const RSS_SUBJECT = 'Subject';
const RSS_ITEM_ISREAD = 'IsRead';
const RSS_AUTHOR = 'Author';

const CDATA_START_TAG = '<![CDATA[';
const CDATA_END_TAG = ']]>';
type
	PTList = ^TList;


/// ܿ  Ʈ   ü
type
	TRSSItem = record
  	strTitle : String;
    strLink : string;
    strDescription : string;
    strPubDate : string;
    strSubject : string;
    bIsRead : Boolean;
end;

type
	PTRSSItem = ^TRSSItem;

/// Ʈ  Ʈ ü
type
	TRSSSite = record
  	title : String;
    link : String;
    rssurl : String;
    description : String;
    itemList : TList;
end;

type
	PTRSSSite = ^TRSSSite;


type
	TCtxPuppyReader = class
  private
	/// Ʈ ǥ Ʈ Ʈ
	m_lstSites: TList;
	m_pCurrentSite : PTRSSSite;
	m_thread : TThread;
//	m_httpConn: TIdHTTP;
  m_XMLDoc: TXMLDocument;
  m_xmlSites : TXMLDocument;
  m_Owner : TComponent;
  m_bBuildXmlListFromLocalCompleted: Boolean;
  m_bBuildXmlListFromNetCompleted: Boolean;
  m_crit : TCriticalSection;
	private
  	function strGetCacheFileName( strUrl:String) : String;
    function iFreeList( argList : TList) : Integer;
		function iLoadSubscribeTree : Integer;
		function iInitItems(pSite:PTRSSSite; xmlNode : IXMLNode):Integer;
    function strParseCdataSection( strXml:String):String;
		procedure vLoadCache(strUrl : String);
    function iSaveSubscribeTree : Integer;
    procedure vParseRSSXmlBuff(strBuff:String;strRssUrl:String;pRssSite : PTRSSSite);
    procedure vParseRSSXmlFile(strFileName:String;pRssSite:PTRSSSite);
    function pFindSite( strRssUrl:String):PTRSSSite;
    function pNewSiteContext(): PTRSSSite;
    procedure vBuildXMLTreeRss(xmlNode : IXMLNode; pSite :PTRSSSite);
    procedure vBuildXMLTreeFile(xmlNode : IXMLNode;pSite :PTRSSSite);
    procedure vSaveCurrentRssItems;
    procedure vGettingRssCompleted(strBuff:String;strUrl:String);
		function bItemsExist( xmlNode : IXMLNode ):Boolean;
    procedure vGetChannelInfo( pSite:PTRSSSite; xmlNode :IXMLNode);
		function bAddArticleToSite(pSite: PTRSSSite; pArticle:PTRSSItem):Boolean;
  public
    constructor Create(AOwner:TComponent);
    destructor Destroy;     override;

    function pAddCurrentSite: PTRSSSite;
    function bExistSite( rssurl : String):Boolean;
  	function stGetCurrentSite : PTRSSSite;

    function rGetSiteLists: TList;
    function bBuildXmlListFromNetCompleted: Boolean;
    function bBuildXmlListFromLocalCompleted: Boolean;
    function strFormatUrl( url: String): String;
    function strGetCurrentRssUrl:String;
    procedure vStartNavigation(strUrl : String);
    function pGetCurrentSite: PTRSSSite;


end;

{TCommThread}
type
	TCommThread = class(TThread)
  private
  	m_pCtx:TCtxPuppyReader;
    m_httpConn : TIdHttp;
    m_strReceived : String;
    m_strUrl : String;
  protected
  	constructor Create( bCreateSuspended: Boolean; pCtx: TCtxPuppyReader;strUrl:String );
  	procedure Execute; override;
    destructor Destroy;
  end;

implementation

constructor TCommThread.Create(bCreateSuspended:Boolean; pCtx: TCtxPuppyReader; strUrl : String);
begin
	inherited Create(bCreateSuspended);
	m_pCtx := pCtx;
  m_strUrl := strUrl;
 	m_httpConn := TIdHttp.Create(nil);
end;

destructor TCommThread.Destroy;
begin
	m_httpConn.Free;
end;

procedure TCommThread.Execute;
begin
	CoInitialize(nil);
  m_strReceived := '';
  try
	  m_strReceived := m_httpConn.Get(m_strUrl);
		m_pCtx.vGettingRssCompleted(m_strReceived,m_strUrl);
  except
  	exit;
  end;
	CoUnInitialize;
end;

constructor TCTxPuppyReader.Create(AOwner:TComponent);
begin
	CoInitialize(nil);
  m_crit := TCriticalSection.Create;
	m_Owner:=AOwner;
  m_XMLDoc := TXmlDocument.Create(m_Owner);
	m_XmlDoc.Options := [doNodeAutoIndent];
  m_xmlSites := TXmlDocument.Create(m_Owner);
	m_XmlSites.Options := [doNodeAutoIndent];
	m_lstSites := TList.Create;

	if DirectoryExists(CACHE_DIR) = false then
  begin
  	CreateDir(CACHE_DIR);
  end;

  iLoadSubscribeTree;
end;

destructor TCtxPuppyReader.Destroy;
begin
  m_XMLDoc.Free;
	m_xmlSites.Active:= False;
  m_xmlSites.Free;
  iFreeList(m_lstSites);
	m_lstSites.Free;
 	CoUnInitialize;
  inherited Destroy;
end;

function TCtxPuppyReader.bBuildXmlListFromLocalCompleted: Boolean;
begin
	bBuildXmlListFromLocalCompleted := m_bBuildXmlListFromLocalCompleted;
end;

function TCTxPuppyReader.bBuildXmlListFromNetCompleted: Boolean;
begin
	bBuildXmlListFromNetCompleted := m_bBuildXmlListFromNetCompleted;
end;


function TCtxPuppyReader.rGetSiteLists: TList;
begin
	rGetSiteLists := m_lstSites;
end;

function TCtxPuppyReader.strGetCacheFileName(strUrl:String) : String;
var
  strTmp : String;
begin
	strTmp := LowerCase( strUrl );
	strTmp := AnsiReplaceStr(strTmp,'http://','');
	strTmp := AnsiReplaceStr(strTmp,'/','_');
	strTmp := AnsiReplaceStr(strTmp,'~','_');
  strTmp := AnsiReplaceStr(strTmp,'.','_');
  strTmp := AnsiReplaceStr(strTmp,'=','_');
	strTmp := AnsiReplaceStr(strTmp,'?','_');
	strGetCacheFileName := strTmp + '.xml';
end;

function TCtxPuppyReader.iLoadSubscribeTree : Integer;
var
	iIndex :Integer;
  tmpNode : IXMLNode;
  pRSSSite : PTRSSSite;
  pItemList : PTList;
begin
	iFreeList(m_lstSites);
	m_xmlSites.Active := false;
  m_xmlSites.XML.Text := '';
  m_xmlSites.Active := true;
  m_xmlSites.LoadFromFile(SITES_FILE);

  for iIndex := 0 to m_xmlSites.DocumentElement.ChildNodes.Count -1 do
  begin
   	tmpNode := m_xmlSites.DocumentElement.ChildNodes.Nodes[iIndex];
    New(pRSSSite);
    pRSSSite.title := tmpNode.Attributes['title'];
    pRSSSite.link := tmpNode.Attributes['link'];
    pRSSSite.rssurl := tmpNode.Attributes['rssurl'];
    pRSSSite.description := tmpNode.Attributes['description'];
    pRssSite.itemList := TList.Create;
    m_lstSites.Add(pRSSSite);
  end;
  iLoadSubscribeTree := 0;
end;

function TCtxPuppyReader.pAddCurrentSite: PTRSSSite;
var
  pItem : PTRSSSite;
begin
  New(pItem);
  pItem.title := m_pCurrentSite.title;
  pItem.link := m_pCurrentSite.link;
  pItem.rssurl := m_pCurrentSite.rssurl;
  pItem.description := m_pCurrentSite.description;
  pItem.itemList := TList.Create;
  m_lstSites.Add(pItem);

  iSaveSubscribeTree;

	pAddCurrentSite := pItem;
end;

function TCtxPuppyReader.iSaveSubscribeTree : Integer;
var
	iIndex :Integer;
  tmpNode : IXMLNode;
begin
	  m_xmlSites.Active := false;
    m_xmlSites.XML.Text := '';
    m_xmlSites.Active := true;
   	m_XmlSites.Options := [doNodeAutoIndent];
		m_xmlSites.DocumentElement := m_xmlSites.CreateNode(ROOT_NODE);

		for iIndex := 0 to m_lstSites.Count-1 do
  	begin
    	tmpNode := m_xmlSites.DocumentElement.AddChild(ITEM_NODE);
			tmpNode.Attributes['title'] := PTRSSSite(m_lstSites.Items[iIndex]).title;
      tmpNode.Attributes['link'] := PTRSSSite(m_lstSites.Items[iIndex]).link;
      tmpNode.Attributes['description'] := PTRSSSite(m_lstSites.Items[iIndex]).description	;
      tmpNode.Attributes['rssurl'] := PTRSSSite(m_lstSites.Items[iIndex]).rssurl;
  	end;
    m_xmlSites.SaveToFile(SITES_FILE);

end;

function TCtxPuppyReader.stGetCurrentSite : PTRSSSite;
begin
	stGetCurrentSite := m_pCurrentSite;
end;

function TCtxPuppyReader.strFormatUrl(url:String):String;
var
	iIdx : Integer;
begin
  iIdx := Pos('http://',url);
  if iIdx=0  then
  begin
  	strFormatUrl := 'http://' + url;
  end
  else
  begin
  	strFormatUrl := url;
  end;
end;

function TCtxPuppyReader.strGetCurrentRssUrl:String;
begin
	strGetCurrentRssUrl := m_pCurrentSite.rssurl;
end;


procedure TCtxPuppyReader.vSaveCurrentRssItems;
var
	iIndex : Integer;
	tmpNode1: IXMLNode;
  tmpNode2: IXMLNode;
  strFileName : String;
  xmlTmp  : TXMLDocument;
begin
  xmlTmp := TXmlDocument.Create(m_Owner);
 	xmlTmp.Options := [doNodeAutoIndent];
  xmlTmp.Active := false;
  xmlTmp.XML.Clear;
  xmlTmp.Active := true;
	xmlTmp.DocumentElement := xmlTmp.CreateNode(RSS_ROOT);
  tmpNode1 := xmlTmp.DocumentElement.AddChild(RSS_TITLE);
  tmpNode1.NodeValue := m_pCurrentSite.title;
	tmpNode1 := xmlTmp.DocumentElement.AddChild(RSS_DESC);
  tmpNode1.NodeValue := m_pCurrentSite.description;
  tmpNode1 := xmlTmp.DocumentElement.AddChild(RSS_WEB_URL);
  tmpNode1.NodeValue := m_pCurrentSite.link;

{	for iIndex := 0 to m_lstItems.Count-1 do
 	begin
   	tmpNode1 := xmlTmp.DocumentElement.AddChild(RSS_ITEM);
    tmpNode2 := tmpNode1.AddChild(RSS_TITLE);
		tmpNode2.NodeValue := PTRSSItem(m_lstItems.Items[iIndex]).strTitle;
    tmpNode2 := tmpNode1.AddChild(RSS_LINK);
		tmpNode2.NodeValue := PTRSSItem(m_lstItems.Items[iIndex]).strLink;
    tmpNode2 := tmpNode1.AddChild(RSS_DESC);
		tmpNode2.NodeValue := PTRSSItem(m_lstItems.Items[iIndex]).strDescription;
    tmpNode2 := tmpNode1.AddChild(RSS_PUBDATE);
		tmpNode2.NodeValue := PTRSSItem(m_lstItems.Items[iIndex]).strPubDate;
 	end;}
	strFileName := strGetCacheFileName(m_pCurrentSite.rssurl);
  strFileNAme := CACHE_DIR + '\' +strFileName;
  xmlTmp.SaveToFile(strFileName );

  xmlTmp.Free;
end;


procedure TCtxPuppyReader.vParseRSSXmlFile(strFileName:String;pRssSite :PTRSSSite);
var
  xmlTmp  : TXMLDocument;
  iIndex : Integer;
  tmpNode : IXMLNode;
begin
  xmlTmp := TXmlDocument.Create(m_Owner);
	xmlTmp.Active := false;
  xmlTmp.LoadFromFile(strFileName);
	xmlTmp.Active := true;

	vBuildXMLTreeFile( xmlTmp.DocumentElement ,pRssSite );
  m_pCurrentSite := pRssSite;
  self.m_bBuildXmlListFromLocalCompleted := true;

  xmlTmp.Free;

end;

procedure TCtxPuppyReader.vBuildXMLTreeFile(xmlNode : IXMLNode;pSite :PTRSSSite);
var
	idx,iItemChildIndex :Integer;
  pRSSItem : PTRSSItem;
  bAdd : Boolean;
  itemNode : IXMLNode;
  iItemStartIndex : Integer;
  iInItemIndex : Integer;
begin
	pRSSItem := nil;
  bAdd := false;

  if pSite = nil then
  begin
  	pSite := pNewSiteContext;
    m_pCurrentSite := pSite;
		m_lstSites.Add(pSite);
  end;

	if xmlNode.NodeName = RSS_ROOT then
  begin
		for idx := 0 to xmlNode.ChildNodes.Count -1 do
    begin
     	if xmlNode.ChildNodes.Get(idx).NodeName = RSS_TITLE then
      begin
       	pSite.title := xmlNode.ChildNodes.Get(idx).Text;
      end
      else if xmlNode.ChildNodes.Get(idx).NodeName = RSS_DESC then
      begin
       	pSite.description := xmlNode.ChildNodes.Get(idx).Text;
      end
      else if xmlNode.ChildNodes.Get(idx).NodeName = RSS_WEB_URL then
      begin
      	pSite.link := xmlNode.ChildNodes.Get(idx).Text;
      end
      else if xmlNode.ChildNodes.Get(idx).NodeName = RSS_ITEM then
      begin
      	itemNode := xmlNode.ChildNodes.Get(idx);
        iItemStartIndex := idx;
      	break;
      end;
	  end;
  end;

  for idx := iItemStartIndex to xmlNode.ChildNodes.Count - 1 do
  begin
		if xmlNode.ChildNodes.Get(idx).NodeName = RSS_ITEM then
    begin
			New(pRssItem);
      itemNode := xmlNode.ChildNodes.Get(idx);
      for iInItemIndex := 0 to itemNode.ChildNodes.Count -1 do
      begin
      	if itemNode.ChildNodes.Get(iInItemIndex).NodeName = RSS_TITLE then
        begin
        	pRssItem.strTitle := itemNode.ChildNodes.Get(iInItemIndex).Text;
        end
        else if itemNode.ChildNodes.Get(iInItemIndex).NodeName = RSS_LINK then
        begin
        	pRssItem.strLink := itemNode.ChildNodes.Get(iInItemIndex).Text;
        end
        else if itemNode.ChildNodes.Get(iInItemIndex).NodeName = RSS_DESC then
        begin
        	pRssItem.strDescription := itemNode.ChildNodes.Get(iInItemIndex).Text;
        end
        else if  not ( Pos(RSS_PUBDATE,Lowercase(itemNode.ChildNodes.Get(iInItemIndex).NodeName) )= 0) then
        begin
        	pRssItem.strPubDate := itemNode.ChildNodes.Get(iInItemIndex).Text;
        end
        else if  not ( Pos(RSS_ITEM_ISREAD,Lowercase(itemNode.ChildNodes.Get(iInItemIndex).NodeName) )= 0) then
        begin
				if itemNode.ChildNodes.Get(iInItemIndex).Text = 'True' then
          pRssItem.bIsRead := True
        else
          pRssItem.bIsRead := False;
        end;
      end;

      if bAddArticleToSite(pSite,pRssItem) = False then
      begin
      	Dispose(pRssItem);
			end;
    end;
  end;
end;

function TCtxPuppyReader.bAddArticleToSite(pSite: PTRSSSite; pArticle:PTRSSItem):Boolean;
var
	iIdx : Integer;
begin
	m_crit.Enter;
  for iIdx := 0 to pSite.itemList.Count - 1 do
  begin
	  if PTRSSItem(pSite.itemList.Items[iIdx]).strLink = pArticle.strLink then
    begin
			bAddArticleToSite := False;
		  m_crit.Leave;
      exit;
    end;
  end;

	pSite.itemList.Add(pArticle);
  bAddArticleToSite := True;
  m_crit.Leave;
end;

procedure TCtxPuppyReader.vParseRSSXMLBuff(strBuff:String;strRssUrl:String;pRssSite:PTRSSSite);
var
	RootElement:IXMLNodeList;
  xmlTmp : TXMLDocument;
begin
	xmlTmp := TXMLDocument.Create(m_owner);
 	xmlTmp.Active := false;
	xmlTmp.XML.SetText(PChar(strBuff));
	xmlTmp.Active := true;

	if not xmlTmp.DocumentElement.IsTextElement then
		RootElement := xmlTmp.DocumentElement.ChildNodes;
	vBuildXMLTreeRss( xmlTmp.DocumentElement , pRssSite );
  m_pCurrentSite := pRssSite;
  m_bBuildXmlListFromNetCompleted := true;

  xmlTmp.Free;
end;


procedure TCtxPuppyReader.vLoadCache(strUrl : String);
var strFileName : String;
begin
	strFileName := self.strGetCacheFileName(strUrl);
	vParseRSSXmlFile(CACHE_DIR +'\' + strFileName,pFindSite(strUrl));

end;

{
	  ׺̼  û.
   Ʈ ׻  ̾.
}
procedure TCtxPuppyReader.vStartNavigation(strUrl : String);
begin
	m_bBuildXmlListFromNetCompleted := false;
  //local data д´.
  // 带  ʿ      ʿ  .
  vLoadCache(strUrl);
  // thread ؼ κ ͸ о´.
 	m_thread :=	TCommThread.Create(True,self,strUrl);
 	m_thread.Resume;
end;

{Context  ϵ ϸ Ƽ  ȯ濡 װ  .}
function TCtxPuppyReader.pNewSiteContext(): PTRSSSite;
var
	pSite : PTRSSSite;
	plstItems: PTList;
begin
	New(pSite);
  pSite.itemList := TList.Create;
	pNewSiteContext := pSite;
end;

procedure TCtxPuppyReader.vGetChannelInfo( pSite:PTRSSSite; xmlNode :IXMLNode);
var
	idx:Integer;
begin
	if xmlNode.NodeName = 'chnnel' then
  begin
		for idx := 0 to xmlNode.ChildNodes.Count -1 do
    begin
     	if xmlNode.ChildNodes.Get(idx).NodeName = 'title' then
      begin
       	pSite.title := xmlNode.ChildNodes.Get(idx).Text;
      end
      else if xmlNode.ChildNodes.Get(idx).NodeName = 'description' then
      begin
       	pSite.description := xmlNode.ChildNodes.Get(idx).Text;
      end
      else if xmlNode.ChildNodes.Get(idx).NodeName = 'link' then
      begin
      	pSite.link := xmlNode.ChildNodes.Get(idx).Text;
      end;
	  end;
  end;

end;

function TCtxPuppyReader.bItemsExist( xmlNode : IXMLNode ):Boolean;
var
	idx:Integer;
begin
	for idx:=0 to xmlNode.ChildNodes.Count -1 do
  begin
		if xmlNode.ChildNodes.Get(idx).NodeName = 'item' then
    begin
    	bItemsExist := true;
      exit;
    end;
  end;
  bItemsExist := false;
end;

function TCtxPuppyReader.strParseCdataSection( strXml:String):String;
var
	iStart :Integer;
  iEnd :Integer;
  iLen : integer;
  strTmp:string;
begin
	iStart := Pos(CDATA_START_TAG,strXml)+StrLen(CDATA_START_TAG);
  iEnd := Pos(CDATA_END_TAG,strXml);
  iLen := StrLen(PChar(strXml) );
	strTmp := MidStr(strXml,iStart,iEnd-iStart);
	strParseCdataSection:=	Trim(strTmp);
end;

function TCtxPuppyReader.iInitItems(pSite:PTRSSSite; xmlNode : IXMLNode):Integer;
var
	idx:Integer;
  pRSSItem :PTRSSItem;
  itemNode,tmpNode :IXMLNode;
  iItemChildIndex :Integer;
  strTmp: String;
  itemList : TList;
begin
	itemList := PSite.itemList;
  for idx := 0 to xmlNode.ChildNodes.Count -1 do
  begin
	  if xmlNode.ChildNodes.Get(idx).NodeName = 'item' then
	  begin
			New(pRSSItem);
			itemNode := xmlNode.ChildNodes.Get(idx);
	  end
    else continue;

		for iItemChildIndex := 0 to itemNode.ChildNodes.Count -1 do
		begin
			tmpNode := itemNode.ChildNodes.Get(iItemChildIndex);

			if not (Pos('title', tmpNode.NodeName ) = 0 ) then
      begin
       	pRSSItem.strTitle := tmpNode.Text;
      end
      else if not (Pos('description',tmpNode.NodeName ) = 0) then
      begin
      	if tmpNode.IsTextElement = true then
        begin
	      	pRssItem.strDescription := tmpNode.Text;
        end
        else begin
	      	pRssItem.strDescription := strParseCdataSection (tmpNode.XML);
        end;
      end
      else if not (Pos('date',Lowercase(tmpNode.NodeName) ) = 0 ) then
      begin
      	pRssItem.strPubDate := tmpNode.Text;
      end
      else if not (Pos('link',tmpNode.NodeName) = 0 ) then
      begin
      	pRssItem.strLink := tmpNode.Text;
      end;
    end;

    if bAddArticleToSite(pSite,pRssItem) = False then
    begin
    	Dispose(pRssItem);
		end;
  end;
end;

{ Network    ؼ ĽϿ  Ʈ Ѵ.}

procedure TCtxPuppyReader.vBuildXMLTreeRss(xmlNode : IXMLNode;pSite :PTRSSSite);
var
	idx,iItemChildIndex :Integer;
  pRSSItem : ^TRSSItem;
  bAdd : Boolean;
  itemNode : IXMLNode;
  iItemStartIndex : Integer;
begin
 	pRSSItem := nil;
  if pSite = nil then
  begin
  	pSite := pNewSiteContext;
		m_lstSites.Add(pSite);
  end;

	if xmlNode.NodeName = 'channel' then
  begin
		vGetChannelInfo(pSite,xmlNode);
  end
  else begin
  	for idx := 0 to xmlNode.ChildNodes.Count - 1 do
    begin
    	if xmlNode.ChildNodes.Get(idx).NodeName = 'channel' then
      begin
      	vGetChannelInfo(pSite,  xmlNode.ChildNodes.Get(idx));
      end;
	    if bItemsExist(xmlNode.ChildNodes.Get(idx)) = true then
	    begin
	    	iInitItems(pSite,xmlNode.ChildNodes.Get(idx));
	    end;
    end;
  end;

  if bItemsExist(xmlNode) = true then
  begin
  	iInitItems(pSite,xmlNode);
  end;

end;

function TCtxPuppyReader.iFreeList( argList : TList) : Integer;
var
	iIndex : Integer;
begin
	for iIndex := 0 to argList.Count - 1 do
  begin
  	Dispose(argList.Items[iIndex]);
  end;
	argList.Clear;
  iFreeList := 0;
end;

function TCtxPuppyReader.bExistSite( rssurl : String):Boolean;
var
	iIndex: Integer;
begin
	for iIndex :=0 to m_lstSites.Count -1 do
  begin
  	if PTRSSSite(m_lstSites.Items[iIndex]).rssurl = rssurl then
    begin
    	bExistSite := true;
      Exit;
    end;
  end;

  bExistSite := false;
end;


{
	 忡   callback ȣϴ Լ ̴.
  strBuff :   ۿ о 
  strUrl : request û url
}
procedure TCtxPuppyReader.vGettingRssCompleted(strBuff:String;strUrl:String);
begin
	vParseRSSXMLBuff(strBuff,strUrl,pFindSite(strUrl));
end;


{ url key Site Site Context . }
function TCtxPuppyReader.pFindSite( strRssUrl:String):PTRSSSite;
var
	iIdx : Integer;
begin
	m_crit.Enter;
	for iIdx := 0 to m_lstSites.Count -1 do
  begin
  	if PTRSSSite(m_lstSites.Items[iIdx]).rssurl = strRssUrl then
    begin
    	pFindSite := m_lstSites.Items[iIdx];
      m_crit.Leave;
      exit;
    end;
  end;
	pFindSite := nil;
  m_crit.Leave;
end;


function TCtxPuppyReader.pGetCurrentSite: PTRSSSite;
begin
	pGetCurrentSite := m_pCurrentSite;
end;

end.




